@@ -1,6 +1,7 @@
 import os
 import json
 from pathlib import Path
+from typing import TextIO
 from urllib.parse import urlparse
 
 from .mixin.admin import AdminMixin
@@ -8,10 +9,13 @@ from .mixin.auth import AuthMixin
 from .mixin.debug import DebugMixin
 from .mixin.enterprise import EnterpriseMixin
 from .mixin.health import HealthMixin
+from .mixin.identity import IdentityMixin
+from .mixin.license import LicenseMixin
 from .mixin.pfs import PFSMixin
 from .mixin.pps import PPSMixin
 from .mixin.transaction import TransactionMixin
 from .mixin.version import VersionMixin
+from .service import Service
 
 
 class ConfigError(Exception):
@@ -42,22 +46,41 @@ class Client(
     DebugMixin,
     EnterpriseMixin,
     HealthMixin,
+    IdentityMixin,
+    LicenseMixin,
     PFSMixin,
     PPSMixin,
     TransactionMixin,
     VersionMixin,
     object,
 ):
+    """The :class:`.Client` class that users will primarily interact with.
+    Initialize an instance with ``python_pachyderm.Client()``.
+
+    To see documentation on the methods :class:`.Client` can call, refer to the
+    `mixins` module.
+    """
+
+    # Class variables for checking config
+    env_config = "PACH_CONFIG"
+    spout_config = "/pachctl/config.json"
+    local_config = f"{Path.home()}/.pachyderm/config.json"
+
     def __init__(
         self,
-        host=None,
-        port=None,
-        auth_token=None,
-        root_certs=None,
-        transaction_id=None,
-        tls=None,
+        host: str = None,
+        port: int = None,
+        auth_token: str = None,
+        root_certs: bytes = None,
+        transaction_id: str = None,
+        tls: bool = None,
+        use_default_host: bool = True,
     ):
-        """Creates a Pachyderm client.
+        """
+        Creates a Pachyderm client. If host and port are unset, checks the
+        ``PACH_CONFIG`` env var for a path. If that's unset, it checks two
+        file paths for a config file. If both files don't exist, a client
+        with default settings is created.
 
         Parameters
         ----------
@@ -76,8 +99,33 @@ class Client(
         tls : bool, optional
             Whether TLS should be used. If `root_certs` are specified, they are
             used. Otherwise, we use the certs provided by certifi.
+        use_default_host : bool, optional
+            Whether to replicate `pachctl` behavior of searching for config.
+
+        Examples
+        --------
+        >>> client = python_pachyderm.Client()
+        ...
+        >>> # Manually set host and port
+        >>> client = python_pachyderm.Client("pachd.example.com", 12345)
         """
 
+        # replicate pachctl behavior to searching for config
+        # if host and port are unset
+        if host is None and port is None and use_default_host:
+            config = Client._check_for_config()
+
+            if config is not None:
+                (
+                    host,
+                    port,
+                    _,
+                    auth_token,
+                    root_certs,
+                    transaction_id,
+                    tls,
+                ) = Client._parse_config(config)
+
         host = host or "localhost"
         port = port or 30650
 
@@ -105,7 +153,9 @@ class Client(
             self._metadata = self._build_metadata()
 
     @classmethod
-    def new_in_cluster(cls, auth_token=None, transaction_id=None):
+    def new_in_cluster(
+        cls, auth_token: str = None, transaction_id: str = None
+    ) -> "Client":
         """Creates a Pachyderm client that operates within a Pachyderm cluster.
 
         Parameters
@@ -120,6 +170,11 @@ class Client(
         -------
         Client
             A python_pachyderm client instance.
+
+        Examples
+        --------
+        >>> from python_pachyderm import Client
+        >>> client = Client.new_in_cluster()
         """
 
         if (
@@ -138,13 +193,21 @@ class Client(
             port = int(os.environ["PACHD_SERVICE_PORT"])
 
         return cls(
-            host=host, port=port, auth_token=auth_token, transaction_id=transaction_id
+            host=host,
+            port=port,
+            auth_token=auth_token,
+            transaction_id=transaction_id,
+            use_default_host=False,
         )
 
     @classmethod
     def new_from_pachd_address(
-        cls, pachd_address, auth_token=None, root_certs=None, transaction_id=None
-    ):
+        cls,
+        pachd_address: str,
+        auth_token: str = None,
+        root_certs: bytes = None,
+        transaction_id: str = None,
+    ) -> "Client":
         """Creates a Pachyderm client from a given pachd address.
 
         Parameters
@@ -164,8 +227,143 @@ class Client(
         -------
         Client
             A python_pachyderm client instance.
+
+        Examples
+        --------
+        >>> from python_pachyderm import Client
+        >>> client = Client.new_from_pachd_address("grpc://pachyderm.com:80/")
+        ...
+        >>> client = Client.new_from_pachd_address("https://pachyderm.com:80", root_certs=b"foo")
+
+        .. # noqa: W505
         """
 
+        u = Client._parse_address(pachd_address)
+
+        return cls(
+            host=u.hostname,
+            port=u.port,
+            auth_token=auth_token,
+            root_certs=root_certs,
+            transaction_id=transaction_id,
+            tls=u.scheme == "grpcs" or u.scheme == "https",
+            use_default_host=False,
+        )
+
+    @classmethod
+    def new_from_config(cls, config_file: TextIO) -> "Client":
+        """Creates a Pachyderm client from a config file-like object.
+
+        Parameters
+        ----------
+        config_file : TextIO
+            A file-like object containing the config json file.
+
+        Returns
+        -------
+        Client
+            A python_pachyderm client instance.
+
+        Examples
+        --------
+        >>> from python_pachyderm import Client
+        >>> config = '''{
+        ...   "v2": {
+        ...     "active_context": "local",
+        ...     "contexts": {
+        ...       "local": {
+        ...         "pachd_address": "grpcs://172.17.0.6:30650",
+        ...         "server_cas": "foo",
+        ...         "session_token": "bar",
+        ...         "active_transaction": "baz"
+        ...       }
+        ...     }
+        ...   }
+        ... }'''
+        >>> client = Client.new_from_config(io.StringIO(config))
+        """
+
+        if config_file is None:
+            raise ConfigError("no config object provided")
+
+        config = json.load(config_file)
+        (
+            _,
+            _,
+            pachd_address,
+            auth_token,
+            root_certs,
+            transaction_id,
+            _,
+        ) = cls._parse_config(config)
+
+        client = cls.new_from_pachd_address(
+            pachd_address,
+            auth_token=auth_token,
+            root_certs=root_certs,
+            transaction_id=transaction_id,
+        )
+
+        context = cls._get_active_context(config)
+        expected_deployment_id = context.get("cluster_deployment_id")
+        if expected_deployment_id:
+            cluster_info = client.inspect_cluster()
+            if cluster_info.deployment_id != expected_deployment_id:
+                raise BadClusterDeploymentID(
+                    expected_deployment_id, cluster_info.deployment_id
+                )
+
+        return client
+
+    @staticmethod
+    def _check_for_config():
+        """Checks for Pachyderm config file locally."""
+
+        j = Client._check_pach_config_env_var()
+        if j is not None:
+            return j
+
+        j = Client._check_pach_config_spout()
+        if j is not None:
+            return j
+
+        j = Client._check_pach_config_local()
+        if j is not None:
+            return j
+
+        print("no config found, proceeding with default behavior")
+
+        return j
+
+    @staticmethod
+    def _check_pach_config_env_var():
+        j = None
+        if Client.env_config in os.environ:
+            with open(os.environ.get(Client.env_config), "r") as config_file:
+                j = json.load(config_file)
+
+        return j
+
+    @staticmethod
+    def _check_pach_config_spout():
+        j = None
+        if os.path.isfile(Client.spout_config):
+            with open(Client.spout_config, "r") as config_file:
+                j = json.load(config_file)
+
+        return j
+
+    @staticmethod
+    def _check_pach_config_local():
+        j = None
+        if os.path.isfile(Client.local_config):
+            with open(Client.local_config, "r") as config_file:
+                j = json.load(config_file)
+
+        return j
+
+    @staticmethod
+    def _parse_address(pachd_address):
         if "://" not in pachd_address:
             pachd_address = "grpc://{}".format(pachd_address)
 
@@ -178,92 +376,46 @@ class Client(
         if u.username is not None or u.password is not None:
             raise ValueError("invalid pachd address")
 
-        return cls(
-            host=u.hostname,
-            port=u.port,
-            auth_token=auth_token,
-            root_certs=root_certs,
-            transaction_id=transaction_id,
-            tls=u.scheme == "grpcs" or u.scheme == "https",
-        )
-
-    @classmethod
-    def new_from_config(cls, config_file=None):
-        """Creates a Pachyderm client from a config file, which can either be
-        passed in as a file-like object, or if unset, checks the PACH_CONFIG env
-        var for a path. If that's also unset, it defaults to loading from
-        '~/.pachyderm/config.json'.
-
-        Parameters
-        ----------
-        config_file : TextIO, optional
-            A file-like object containing the config json file. If unspecified,
-            we load the config from the default location
-            ('~/.pachyderm/config.json').
-
-        Returns
-        -------
-        Client
-            A python_pachyderm client instance.
-        """
-
-        if config_file is not None:
-            j = json.load(config_file)
-        elif "PACH_CONFIG" in os.environ:
-            with open(os.environ.get("PACH_CONFIG"), "r") as config_file:
-                j = json.load(config_file)
-                print("config: {}".format(str(j)))
-        else:
-            try:
-                # Search for config file in default home location
-                with open(
-                    str(Path.home() / ".pachyderm/config.json"), "r"
-                ) as config_file:
-                    j = json.load(config_file)
-            except FileNotFoundError:
-                # If not found, search in "/pachctl" (default mount for spout)
-                with open("/pachctl/config.json", "r") as config_file:
-                    j = json.load(config_file)
+        return u
 
+    @staticmethod
+    def _get_active_context(config):
         try:
-            active_context = j["v2"]["active_context"]
+            active_context = config["v2"]["active_context"]
         except KeyError:
             raise ConfigError("no active context")
 
         try:
-            context = j["v2"]["contexts"][active_context]
+            context = config["v2"]["contexts"][active_context]
         except KeyError:
             raise ConfigError("missing active context '{}'".format(active_context))
 
+        return context
+
+    @staticmethod
+    def _parse_config(config):
+        context = Client._get_active_context(config)
+
         auth_token = context.get("session_token")
         root_certs = context.get("server_cas")
         transaction_id = context.get("active_transaction")
 
         pachd_address = context.get("pachd_address")
-        if pachd_address:
-            client = cls.new_from_pachd_address(
-                pachd_address,
-                auth_token=auth_token,
-                root_certs=root_certs,
-                transaction_id=transaction_id,
-            )
-        else:
+        if not pachd_address:
             port_forwarders = context.get("port_forwarders", {})
             pachd_port = port_forwarders.get("pachd", 30650)
             pachd_address = "grpc://localhost:{}".format(pachd_port)
-            client = cls.new_from_pachd_address(
-                pachd_address, auth_token=auth_token, transaction_id=transaction_id
-            )
 
-        expected_deployment_id = context.get("cluster_deployment_id")
-        if expected_deployment_id:
-            cluster_info = client.inspect_cluster()
-            if cluster_info.deployment_id != expected_deployment_id:
-                raise BadClusterDeploymentID(
-                    expected_deployment_id, cluster_info.deployment_id
-                )
+            root_certs = None
 
-        return client
+        u = Client._parse_address(pachd_address)
+
+        host = u.hostname
+        port = u.port
+        tls = u.scheme == "grpcs" or u.scheme == "https"
+        root_certs = bytes(root_certs, "utf-8") if root_certs is not None else None
+
+        return host, port, pachd_address, auth_token, root_certs, transaction_id, tls
 
     @property
     def auth_token(self):
@@ -291,16 +443,23 @@ class Client(
             metadata.append(("pach-transaction", self._transaction_id))
         return metadata
 
-    def _req(self, grpc_service, grpc_method_name, req=None, **kwargs):
+    def _req(self, grpc_service: Service, grpc_method_name, req=None, **kwargs):
         stub = self._stubs.get(grpc_service)
         if stub is None:
             grpc_module = grpc_service.grpc_module
             if self.root_certs:
                 ssl_channel_credentials = grpc_module.grpc.ssl_channel_credentials
                 ssl = ssl_channel_credentials(root_certificates=self.root_certs)
-                channel = grpc_module.grpc.secure_channel(self.address, ssl)
+                channel = grpc_module.grpc.secure_channel(
+                    self.address,
+                    ssl,
+                    options=grpc_service.options,
+                )
             else:
-                channel = grpc_module.grpc.insecure_channel(self.address)
+                channel = grpc_module.grpc.insecure_channel(
+                    self.address,
+                    options=grpc_service.options,
+                )
             stub = grpc_service.stub(channel)
             self._stubs[grpc_service] = stub
 
@@ -318,3 +477,11 @@ class Client(
 
         grpc_method = getattr(stub, grpc_method_name)
         return grpc_method(req, metadata=self._metadata)
+
+    def delete_all(self) -> None:
+        """Delete all repos, commits, files, pipelines, and jobs. This resets
+        the cluster to its initial state.
+        """
+        self.delete_all_pipelines()
+        self.delete_all_repos()
+        self.delete_all_transactions()
