@@ -1,4 +1,7 @@
 import io
+import os
+import json
+from pathlib import Path
 
 import pytest
 import python_pachyderm
@@ -7,11 +10,165 @@ from tests import util
 """Tests generic client functionality"""
 
 
-def test_client_init_with_default_host_port():
+def test_check_config_order(mocker):
+    env_config = json.loads(
+        """
+      {
+        "v2": {
+          "active_context": "local",
+          "contexts": {
+            "local": {
+              "pachd_address": "grpcs://172.17.0.6:30650",
+              "server_cas": "foo",
+              "session_token": "bar",
+              "active_transaction": "baz"
+            }
+          }
+        }
+      }
+    """
+    )
+    spout_config = json.loads(
+        """
+      {
+        "v2": {
+          "active_context": "local",
+          "contexts": {
+            "local": {
+              "pachd_address": "[::1]:80",
+              "server_cas": "foo",
+              "session_token": "bar",
+              "active_transaction": "baz"
+            }
+          }
+        }
+      }
+    """
+    )
+    local_config = json.loads(
+        """
+      {
+        "v2": {
+          "active_context": "local",
+          "contexts": {
+            "local": {
+              "port_forwarders": {
+                "pachd": 10101
+              }
+            }
+          }
+        }
+      }
+    """
+    )
+
+    mocker.patch(
+        "python_pachyderm.Client._check_pach_config_env_var", return_value=env_config
+    )
+    mocker.patch(
+        "python_pachyderm.Client._check_pach_config_spout",
+        return_value=spout_config,
+    )
+    mocker.patch(
+        "python_pachyderm.Client._check_pach_config_local",
+        return_value=local_config,
+    )
+
+    # Retrieves from env var config file
+    client = python_pachyderm.Client()
+    assert client.address == "172.17.0.6:30650"
+
+    # Retrieves from spout config file
+    mocker.patch(
+        "python_pachyderm.Client._check_pach_config_env_var", return_value=None
+    )
+    client = python_pachyderm.Client()
+    assert client.address == "::1:80"
+
+    # Retrieves from local config file
+    mocker.patch("python_pachyderm.Client._check_pach_config_spout", return_value=None)
+    client = python_pachyderm.Client()
+    assert client.address == "localhost:10101"
+
+    # Executes default constructor behavior
+    mocker.patch("python_pachyderm.Client._check_pach_config_local", return_value=None)
     client = python_pachyderm.Client()
     assert client.address == "localhost:30650"
 
 
+def test_parse_config():
+    (
+        host,
+        port,
+        pachd_address,
+        auth_token,
+        root_certs,
+        transaction_id,
+        tls,
+    ) = python_pachyderm.Client._parse_config(
+        json.loads(
+            """
+        {
+          "v2": {
+            "active_context": "local",
+            "contexts": {
+              "local": {
+                "pachd_address": "grpcs://172.17.0.6:30650",
+                "server_cas": "foo",
+                "session_token": "bar",
+                "active_transaction": "baz"
+              }
+            }
+          }
+        }
+        """
+        )
+    )
+    assert host == "172.17.0.6"
+    assert port == 30650
+    assert pachd_address == "grpcs://172.17.0.6:30650"
+    assert auth_token == "bar"
+    assert root_certs == b"foo"
+    assert transaction_id == "baz"
+    assert tls is True
+
+    (
+        host,
+        port,
+        pachd_address,
+        auth_token,
+        root_certs,
+        transaction_id,
+        tls,
+    ) = python_pachyderm.Client._parse_config(
+        json.loads(
+            """
+        {
+          "v2": {
+            "active_context": "local",
+            "contexts": {
+              "local": {
+                "port_forwarders": {
+                  "pachd": 10101
+                },
+                "server_cas": "foo",
+                "session_token": "bar"
+              }
+            }
+          }
+        }
+    """
+        )
+    )
+    assert host == "localhost"
+    assert port == 10101
+    assert pachd_address == "grpc://localhost:10101"
+    assert auth_token == "bar"
+    assert root_certs is None
+    assert transaction_id is None
+    assert tls is False
+
+
 def test_client_init_with_args():
     client = python_pachyderm.Client(host="pachd.example.com", port=54321)
     assert client.address == "pachd.example.com:54321"
@@ -19,7 +176,7 @@ def test_client_init_with_args():
 
 def test_client_new_in_cluster_missing_envs():
     with pytest.raises(Exception):
-        client = python_pachyderm.Client.new_in_cluster()
+        python_pachyderm.Client.new_in_cluster()
 
 
 def test_client_new_from_pachd_address():
@@ -134,7 +291,7 @@ def test_client_new_from_config():
         )
     )
     assert client.address == "172.17.0.6:30650"
-    assert client.root_certs == "foo"
+    assert client.root_certs == b"foo"
     assert client.auth_token == "bar"
     assert client.transaction_id == "baz"
 
@@ -159,7 +316,7 @@ def test_client_new_from_config():
     )
     assert client.address == "localhost:10101"
 
-    # empty context should default ot localhost:30650
+    # empty context should default to localhost:30650
     client = python_pachyderm.Client.new_from_config(
         config_file=io.StringIO(
             """
